(* Graph, a terminal time series graphing utility
   Copyright (C) 2007 N. Owen Gunden

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. *)

open Printf

let parse_args () =
  let anonargs = ref [] in
  let anonfun a = anonargs := a :: !anonargs in
  let usage_msg = Printf.sprintf "%s <min> <max>" Sys.argv.(0) in
  let delay = ref 1. in
  let mode = ref `totally_random in
  let specl = Arg.align [
    "-delay", Arg.Set_float delay, sprintf " delay between printing random numbers (%f)" !delay;
    "-walk", Arg.Unit (fun () -> mode := `walk), " random walk mode";
  ] in
  Arg.parse specl anonfun usage_msg;
  match List.rev !anonargs with
  | [min; max] -> (!mode, int_of_string min, int_of_string max, !delay)
  | _ -> (Arg.usage specl usage_msg; exit 1)

let pause d =
  try
    ignore (Unix.select [] [] [] d);
  with _ -> ()

let () =
  Random.self_init ();
  let mode, min, max, delay = parse_args () in
  match mode with
  | `totally_random ->
    while true do
      let v = Random.int (max - min) + min in
      Printf.printf "%d\n%!" v;
      pause delay
    done
  | `walk ->
    let last = ref (Random.int (max - min) + min) in
    while true do
      Printf.printf "%d\n%!" !last;
      (if Random.bool () then
        if !last = max then () else incr last
      else
        if !last = min then () else decr last);
      pause delay
    done
